#ifndef __EDG_WORKLOAD_LOGGING_CLIENT_EVENTS_H__
#define __EDG_WORKLOAD_LOGGING_CLIENT_EVENTS_H__

/**
 * \brief contains definition of event type codes for use both by lbapi.h and dglog.h
 */

#ident "$Header: /cvs/glite/org.glite.lb.client-interface/interface/events.h.T,v 1.6 2007/02/06 11:27:40 mmulac Exp $"
/*

  !! Automatically generated file
  !! Do not edit, your changes will be discarded upon build
  !! Change the corresponding template file ../interface/events.h.T

*/

#include <sys/time.h>
#include <inttypes.h>

#include "glite/wmsutils/jobid/cjobid.h"
#include "glite/lb/context.h"
#include "glite/lb/notifid.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * \defgroup events Events
 * \brief All L&B event types.
 *
 *@{
 */

/**
 * Predefined type for ULM string 
 */
typedef char *edg_wll_LogLine;


/**
 * \typedef edg_wll_EventCode
 * Predefined event types 
 */
typedef enum _edg_wll_EventCode {
/** invalid code, e.g. uninitialized variable */
        EDG_WLL_EVENT_UNDEF = 0,
	EDG_WLL_EVENT_TRANSFER,	/**< Start, success, or failure of job transfer to another component. */
	EDG_WLL_EVENT_ACCEPTED,	/**< Accepting job (successful counterpart to Transfer). */
	EDG_WLL_EVENT_REFUSED,	/**< Refusing job (unsuccessful counterpart to Transfer). */
	EDG_WLL_EVENT_ENQUEUED,	/**< The job has been enqueued in an inter-component queue. */
	EDG_WLL_EVENT_DEQUEUED,	/**< The job has been dequeued from an inter-component queue. */
	EDG_WLL_EVENT_HELPERCALL,	/**< Helper component is called. */
	EDG_WLL_EVENT_HELPERRETURN,	/**< Helper component is returning the control. */
	EDG_WLL_EVENT_RUNNING,	/**< Job wrapper started. */
	EDG_WLL_EVENT_RESUBMISSION,	/**< Result of resubmission decision. */
	EDG_WLL_EVENT_DONE,	/**< Execution terminated (normally or abnormally). */
	EDG_WLL_EVENT_CANCEL,	/**< Cancel operation has been attempted on the job. */
	EDG_WLL_EVENT_ABORT,	/**< Job aborted by system. */
	EDG_WLL_EVENT_CLEAR,	/**< Job cleared, output sandbox removed */
	EDG_WLL_EVENT_PURGE,	/**< Job is purged from bookkepping server. */
	EDG_WLL_EVENT_MATCH,	/**< Matching CE found. */
	EDG_WLL_EVENT_PENDING,	/**< No matching CE found yet. */
	EDG_WLL_EVENT_REGJOB,	/**< New job registration. */
	EDG_WLL_EVENT_CHKPT,	/**< Application-specific checkpoint record. */
	EDG_WLL_EVENT_LISTENER,	/**< Listening network port for interactive control. */
	EDG_WLL_EVENT_CURDESCR,	/**< Current state of job processing (optional event). */
	EDG_WLL_EVENT_USERTAG,	/**< User tag -- arbitrary name=value pair. */
	EDG_WLL_EVENT_CHANGEACL,	/**< Management of ACL stored on bookkepping server. */
	EDG_WLL_EVENT_NOTIFICATION,	/**< Management of notification service. */
	EDG_WLL_EVENT_RESOURCEUSAGE,	/**< Resource (CPU, memory etc.) consumption. */
	EDG_WLL_EVENT_REALLYRUNNING,	/**< User payload started. */
	EDG_WLL_EVENT_SUSPEND,	/**< Job execution (queuing) was suspended. */
	EDG_WLL_EVENT_RESUME,	/**< Job execution (queuing) was resumed. */
	EDG_WLL_EVENT_COLLECTIONSTATE,	/**< State of the collection. */
	EDG_WLL_EVENT_PBSQUEUED,	/**< Job enqued */
	EDG_WLL_EVENT_PBSMATCH,	/**< Scheduler created exec_host */
	EDG_WLL_EVENT_PBSPENDING,	/**< Scheduler is not able to find exec_host, or some error occured */
	EDG_WLL_EVENT_PBSRUN,	/**< Job attempted to be run by the logging component */
	EDG_WLL_EVENT_PBSRERUN,	/**< Job rerun requested */
	EDG_WLL_EVENT_PBSDONE,	/**< Job terminated */
	EDG_WLL_EVENT_PBSDEQUEUED,	/**< Job dequeued */
	EDG_WLL_EVENT_PBSRESOURCEUSAGE,	/**< Resources requested/consumed */
	EDG_WLL_EVENT_PBSERROR,	/**< Any error occured */
	EDG_WLL_EVENT_CONDORMATCH,	/**< Job MATCHed */
	EDG_WLL_EVENT_CONDORREJECT,	/**< Job REJECTed */
	EDG_WLL_EVENT_CONDORRUN,	/**< Condor EXECUTEed */
	EDG_WLL_EVENT_CONDORCHKPT,	/**<  */
	EDG_WLL_EVENT_CONDOREVICT,	/**<  */
	EDG_WLL_EVENT_CONDORTERMINATE,	/**<  */
	EDG_WLL_EVENT_CONDORABORT,	/**<  */
	EDG_WLL_EVENT_CONDORSUSPEND,	/**<  */
	EDG_WLL_EVENT_CONDORUNSUSPEND,	/**<  */
	EDG_WLL_EVENT_CONDORHOLD,	/**<  */
	EDG_WLL_EVENT_CONDORRELEASE,	/**<  */
	EDG_WLL_EVENT_CONDORNODEEXECUTE,	/**<  */
	EDG_WLL_EVENT_CONDORNODETERMINATE,	/**<  */
	EDG_WLL_EVENT_CONDORPOSTSCTIPTTERMINATE,	/**<  */
	EDG_WLL_EVENT_CONDORGLOBUSSUBMIT,	/**<  */
	EDG_WLL_EVENT_CONDORGLOBUSRESOURCE,	/**<  */
        EDG_WLL_EVENT__LAST,	/**< last currently supported event code */
} edg_wll_EventCode;

/**
 * \fn edg_wll_EventCode edg_wll_StringToEvent(const char *name)
 * \param[in] name		a string event name (e.g. "JobTransfer")
 * \return corresponding numeric code (edg_wll_EventCode)
 * \brief convert a string event name to the corresponding numeric code
 */

extern edg_wll_EventCode edg_wll_StringToEvent(const char *name);

/**
 * \fn char *edg_wll_EventToString(edg_wll_EventCode event)
 * \param[in] event		an event numeric code (edg_wll_EventCode)
 * \return corresponding string (e.g. "JobTransfer")
 * \brief convert an event numeric code to the corresponding string
 */

extern char *edg_wll_EventToString(edg_wll_EventCode event);


/**
 * \typedef edg_wll_KeyNameCode
 * Predefined ULM key types
 */
typedef enum _edg_wll_KeyNameCode {
	UNDEFINED,		/**< undefined */
	EDG_WLL_EVNT,		/**< event type */
	ULM_DATE,		/**< Time the event was generated. */
	ULM_ARR_DATE,		/**< Time the event was stored into the bookkeeping server database. */
	ULM_HOST,		/**< Hostname of the machine where the event was generated. */
	ULM_LVL,		/**< Logging level (in the range from DEBUG to EMERGENCY). */
	EDG_WLL_COMMON_PRIORITY,		/**< Message priority (yet 0 for asynchronous and 1 for synchronous transfers). */
	EDG_WLL_COMMON_JOBID,		/**< Grid job id of the job the event belongs to. */
	EDG_WLL_COMMON_SEQCODE,		/**< Sequence code assigned to the event. */
	EDG_WLL_COMMON_USER,		/**< Identity (certificate subject) of the event sender. */
	EDG_WLL_COMMON_SOURCE,		/**< Source (software component) which generated this event. */
	EDG_WLL_COMMON_SRC_INSTANCE,		/**< Instance of source component (e.g. service communication endpoint). */
	EDG_WLL_TRANSFER_DESTINATION,	/**< Destination where the job is being transfered to. */
	EDG_WLL_TRANSFER_DEST_HOST,	/**< Hostname of server that takes over control of the job. */
	EDG_WLL_TRANSFER_DEST_INSTANCE,	/**< Service (instance) that takes over control of the job. */
	EDG_WLL_TRANSFER_JOB,	/**< Job description in receiver's language. */
	EDG_WLL_TRANSFER_RESULT,	/**< Result code of the transfer attempt (START, OK, REFUSED or FAIL). */
	EDG_WLL_TRANSFER_REASON,	/**< Detailed description of the transfer, especially reason of failure. */
	EDG_WLL_TRANSFER_DEST_JOBID,	/**< Job id as assigned by the receiving software component. */
	EDG_WLL_ACCEPTED_FROM,	/**< The software component  the job was received from. */
	EDG_WLL_ACCEPTED_FROM_HOST,	/**< Hostname of the component the job was received from. */
	EDG_WLL_ACCEPTED_FROM_INSTANCE,	/**< Instance of the component the job was received from. */
	EDG_WLL_ACCEPTED_LOCAL_JOBID,	/**< New job id as assigned by the receiving component. */
	EDG_WLL_REFUSED_FROM,	/**< The software component that tried to send the job. */
	EDG_WLL_REFUSED_FROM_HOST,	/**< Hostname of the component that tried to send the job. */
	EDG_WLL_REFUSED_FROM_INSTANCE,	/**< Instance of the component that tried to send the job. */
	EDG_WLL_REFUSED_REASON,	/**< Description of the reason why the job was refused. */
	EDG_WLL_ENQUEUED_QUEUE,	/**< Queue into which the job has been stored for retrieval by another component. */
	EDG_WLL_ENQUEUED_JOB,	/**< Job description in the receiver's language. */
	EDG_WLL_ENQUEUED_RESULT,	/**< Result code of the attempt to put job into the queue (START, OK, REFUSED or FAIL). */
	EDG_WLL_ENQUEUED_REASON,	/**< Detailed description of the attempt to enqueue the job, especially the reason of failure. */
	EDG_WLL_DEQUEUED_QUEUE,	/**< Name of the queue the job was obtained from. */
	EDG_WLL_DEQUEUED_LOCAL_JOBID,	/**< New job id as assigned by the retreiving component. */
	EDG_WLL_HELPERCALL_HELPER_NAME,	/**< Name of the called helper component. */
	EDG_WLL_HELPERCALL_HELPER_PARAMS,	/**< Parameters of the call to the helper component. */
	EDG_WLL_HELPERCALL_SRC_ROLE,	/**< The role the event sender is playing in the helper call (CALLING or CALLEE). */
	EDG_WLL_HELPERRETURN_HELPER_NAME,	/**< Name of the called helper component. */
	EDG_WLL_HELPERRETURN_RETVAL,	/**< Data returned by the call to the helper component. */
	EDG_WLL_HELPERRETURN_SRC_ROLE,	/**< The role the event sender is playing in the helper call (CALLING or CALLEE). */
	EDG_WLL_RUNNING_NODE,	/**< Worker node on which the job executable is being run. */
	EDG_WLL_RESUBMISSION_RESULT,	/**< Result code of the resubmission decision (WILLRESUB or WONTRESUB or SHALLOW). */
	EDG_WLL_RESUBMISSION_REASON,	/**< Reason why the job will or will not be resubmitted. */
	EDG_WLL_RESUBMISSION_TAG,	/**< Value of the attribute on which the decision to resubmit the job was based. */
	EDG_WLL_DONE_STATUS_CODE,	/**< Reason code for the termination of the job (OK, FAILED or CANCELLED). */
	EDG_WLL_DONE_REASON,	/**< Detailed description why the job was terminated. */
	EDG_WLL_DONE_EXIT_CODE,	/**< Exit code of the job's process. */
	EDG_WLL_CANCEL_STATUS_CODE,	/**< Classification of the attempt to cancel the job (REQ, REFUSE, DONE or ABORT). */
	EDG_WLL_CANCEL_REASON,	/**< Detailed description of the attempt to cancel the job, especially the reason of failure. */
	EDG_WLL_ABORT_REASON,	/**< Reason why the job was aborted by the system. */
	EDG_WLL_CLEAR_REASON,	/**< Description of the reason why the job was cleared and the output sandbox removed (USER, TIMEOUT or NOOUTPUT). */
	EDG_WLL_MATCH_DEST_ID,	/**< Identification of the queue on the CE that the job could be send to. */
	EDG_WLL_PENDING_REASON,	/**< Description why the matching CE for the job was not found (yet). */
	EDG_WLL_REGJOB_JDL,	/**< Job description of the job being registered. */
	EDG_WLL_REGJOB_NS,	/**< NetworkServer handling the newly registered job. */
	EDG_WLL_REGJOB_PARENT,	/**< Grid job id of the parent job registering this new one. */
	EDG_WLL_REGJOB_JOBTYPE,	/**< Type of the job being registered (SIMPLE, DAG, PARTITIONABLE or PARTITIONED). */
	EDG_WLL_REGJOB_NSUBJOBS,	/**< Number of subjobs this job plans to spawn. */
	EDG_WLL_REGJOB_SEED,	/**< Seed for subjob id generation. */
	EDG_WLL_CHKPT_TAG,	/**< Application specific checkpoint tag. */
	EDG_WLL_CHKPT_CLASSAD,	/**< Application specific checkpoint value. */
	EDG_WLL_LISTENER_SVC_NAME,	/**< Name of the port instance for interactive job control. */
	EDG_WLL_LISTENER_SVC_HOST,	/**< Hostname of the interactive job controller. */
	EDG_WLL_LISTENER_SVC_PORT,	/**< Port number of the interactive job controller. */
	EDG_WLL_CURDESCR_DESCR,	/**< Description of the current job transformation (output of the helper). */
	EDG_WLL_USERTAG_NAME,	/**< Arbitrary user tag name. */
	EDG_WLL_USERTAG_VALUE,	/**< Arbitrary user tag value. */
	EDG_WLL_CHANGEACL_USER_ID,	/**< DN or VOMS parameter (in format VO:group). */
	EDG_WLL_CHANGEACL_USER_ID_TYPE,	/**< Type of information given in user_id (DN or VOMS). */
	EDG_WLL_CHANGEACL_PERMISSION,	/**< ACL permission to change (currently only READ). */
	EDG_WLL_CHANGEACL_PERMISSION_TYPE,	/**< Type of permission requested ('allow', 'deny'). */
	EDG_WLL_CHANGEACL_OPERATION,	/**< Operation requested to perform with ACL (add, remove). */
	EDG_WLL_NOTIFICATION_NOTIFID,	/**< Notification id. */
	EDG_WLL_NOTIFICATION_OWNER,	/**< Identification of the job owner (certificate subject). */
	EDG_WLL_NOTIFICATION_DEST_HOST,	/**< Hostname the notification is sent to. */
	EDG_WLL_NOTIFICATION_DEST_PORT,	/**< Port number the notification is sent to. */
	EDG_WLL_NOTIFICATION_JOBSTAT,	/**< Status of the job (the notification content). */
	EDG_WLL_RESOURCEUSAGE_RESOURCE,	/**< Resource's name. */
	EDG_WLL_RESOURCEUSAGE_QUANTITY,	/**< Resources's quantity (how much). */
	EDG_WLL_RESOURCEUSAGE_UNIT,	/**< Units (sec, kB, etc.). */
	EDG_WLL_REALLYRUNNING_WN_SEQ,	/**< Sequence code on the worker node. */
	EDG_WLL_SUSPEND_REASON,	/**< Reason for the suspend. */
	EDG_WLL_RESUME_REASON,	/**< Reason for the resume. */
	EDG_WLL_COLLECTIONSTATE_STATE,	/**< New collection state. */
	EDG_WLL_COLLECTIONSTATE_DONE_CODE,	/**< In case of (state == Done) contains done code  */
	EDG_WLL_COLLECTIONSTATE_HISTOGRAM,	/**< User readable histogram; useful for debugging. */
	EDG_WLL_COLLECTIONSTATE_CHILD,	/**< JobId of subjob, which triggered the state change.	 */
	EDG_WLL_COLLECTIONSTATE_CHILD_EVENT,	/**< Event which triggered the state change.	 */
	EDG_WLL_PBSQUEUED_QUEUE,	/**< Queue name */
	EDG_WLL_PBSQUEUED_OWNER,	/**< Job owner */
	EDG_WLL_PBSQUEUED_NAME,	/**< Job name */
	EDG_WLL_PBSMATCH_DEST_HOST,	/**< Aka exec_host */
	EDG_WLL_PBSPENDING_REASON,	/**< Reasons of job pendation or errors */
	EDG_WLL_PBSRUN_SCHEDULER,	/**< Scheduler ID */
	EDG_WLL_PBSRUN_DEST_HOST,	/**< Where to run the job */
	EDG_WLL_PBSRUN_PID,	/**< Actual process ID */
	EDG_WLL_PBSDONE_EXIT_STATUS,	/**< Exit status */
	EDG_WLL_PBSRESOURCEUSAGE_USAGE,	/**< Type of record */
	EDG_WLL_PBSRESOURCEUSAGE_NAME,	/**< Name of resource */
	EDG_WLL_PBSRESOURCEUSAGE_QUANTITY,	/**< The quantity */
	EDG_WLL_PBSRESOURCEUSAGE_UNIT,	/**< Units (sec, kB, etc.) */
	EDG_WLL_PBSERROR_ERROR_DESC,	/**< Error reason */
	EDG_WLL_CONDORMATCH_OWNER,	/**< Owner */
	EDG_WLL_CONDORMATCH_MATCHED_HOST,	/**< Matched host */
	EDG_WLL_CONDORMATCH_PREEMPTING,	/**< Preempting */
	EDG_WLL_CONDORREJECT_OWNER,	/**< Owner */
	EDG_WLL_CONDORREJECT_STATUS_CODE,	/**< Reason code for the rejection of the job */
	EDG_WLL_CONDORRUN_CONDOR_ID,	/**< Condor ID */
	EDG_WLL_CONDORRUN_UNIVERSE,	/**< Condor Universe */
	EDG_WLL_CONDORRUN_SCHEDULER,	/**< Scheduler ID */
	EDG_WLL_CONDORRUN_DEST_HOST,	/**< Where to run the job */
	EDG_WLL_CONDORRUN_PID,	/**< Actual process ID */
	EDG_WLL_CONDORCHKPT_INFO,	/**< Some useful info */
	EDG_WLL_CONDOREVICT_SHADOW_EXIT_STATUS,	/**< Condor shadow exit status */
	EDG_WLL_CONDOREVICT_REASON,	/**< reason */
	EDG_WLL_CONDORTERMINATE_SHADOW_EXIT_STATUS,	/**< Condor shadow exit status */
	EDG_WLL_CONDORTERMINATE_REASON,	/**< reason */
	EDG_WLL_CONDORABORT_SHADOW_EXIT_STATUS,	/**< Condor shadow exit status */
	EDG_WLL_CONDORABORT_REASON,	/**< reason */
	EDG_WLL_CONDORSUSPEND_SHADOW_EXIT_STATUS,	/**< Condor shadow exit status */
	EDG_WLL_CONDORSUSPEND_REASON,	/**< reason */
	EDG_WLL_CONDORUNSUSPEND_SHADOW_EXIT_STATUS,	/**< Condor shadow exit status */
	EDG_WLL_CONDORUNSUSPEND_REASON,	/**< reason */
	EDG_WLL_CONDORHOLD_SHADOW_EXIT_STATUS,	/**< Condor shadow exit status */
	EDG_WLL_CONDORHOLD_REASON,	/**< reason */
	EDG_WLL_CONDORRELEASE_SHADOW_EXIT_STATUS,	/**< Condor shadow exit status */
	EDG_WLL_CONDORRELEASE_REASON,	/**< reason */
	EDG_WLL_CONDORNODEEXECUTE_INFO,	/**< Some useful info */
	EDG_WLL_CONDORNODETERMINATE_INFO,	/**< Some useful info */
	EDG_WLL_CONDORPOSTSCTIPTTERMINATE_INFO,	/**< Some useful info */
	EDG_WLL_CONDORGLOBUSSUBMIT_STATUS_CODE,	/**<  */
	EDG_WLL_CONDORGLOBUSSUBMIT_INFO,	/**< Some useful info */
	EDG_WLL_CONDORGLOBUSRESOURCE_STATUS_CODE,	/**< Status code */
	EDG_WLL_CONDORGLOBUSRESOURCE_INFO,	/**< Some useful info */
	EDG_WLL_INTERNAL_TYPE,		/**< internal message type */
} edg_wll_KeyNameCode;

/**
 * \fn edg_wll_KeyNameCode edg_wll_StringToKeyName(const char *name)
 * \param[in] name		a string ULM key name (e.g. "DG.JOB.TRANSFER.DEST")
 * \return corresponding numeric code (edg_wll_KeyNameCode)
 * \brief convert a string ULM key name to the corresponding numeric code
 */

extern edg_wll_KeyNameCode edg_wll_StringToKeyName(const char *name);

/**
 * \fn char *edg_wll_KeyNameToString(edg_wll_KeyNameCode key)
 * \param[in] key		a ULM key name numeric code (edg_wll_KeyNameCode)
 * \return corresponding string (e.g. "DG.JOB.TRANSFER.DEST")
 * \brief convert a ULM key name numeric code to the corresponding string
 */

extern char *edg_wll_KeyNameToString(edg_wll_KeyNameCode key);


/**
 * Predefined _code_ types and
 * related StringTo_code and _code_ToString function prototypes
 */

/**
 * \enum edg_wll_Level
 * Level codes 
 */
enum edg_wll_Level {
	EDG_WLL_LEVEL_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_LEVEL_EMERGENCY,	/**< emergency */ 
	EDG_WLL_LEVEL_ALERT,	/**< alert */ 
	EDG_WLL_LEVEL_ERROR,	/**< error */ 
	EDG_WLL_LEVEL_WARNING,	/**< warning */ 
	EDG_WLL_LEVEL_AUTH,	/**< authentication */ 
	EDG_WLL_LEVEL_SECURITY,	/**< security */ 
	EDG_WLL_LEVEL_USAGE,	/**< usage */ 
	EDG_WLL_LEVEL_SYSTEM,	/**< system */ 
	EDG_WLL_LEVEL_IMPORTANT,	/**< important */ 
	EDG_WLL_LEVEL_DEBUG,	/**< debug */ 
};

/**
 * \fn enum edg_wll_Level edg_wll_StringToLevel(const char *name);
 * \param[in] name		a string representing Level code (e.g. "ALERT")
 * \return corresponding numeric code (enum edg_wll_Level)
 * \brief converts a string Level code to corresponding numeric code
 */
extern enum edg_wll_Level edg_wll_StringToLevel(const char *name);

/**
 * \fn char *edg_wll_LevelToString(enum edg_wll_Level code);
 * \param[in] code		a Level numeric code (enum edg_wll_Level)
 * \return corresponding string (e.g. "ALERT")
 * \brief converts a Level numeric code to corresponding string
 */
extern char *edg_wll_LevelToString(enum edg_wll_Level code);


/**
 * \enum edg_wll_TransferResult
 * Result codes of the Transfer event
 */
enum edg_wll_TransferResult {
	EDG_WLL_TRANSFER_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_TRANSFER_START,	/**< The sending component has started or is about to start the transfer. */ 
	EDG_WLL_TRANSFER_OK,	/**< The job was sent successfully. */ 
	EDG_WLL_TRANSFER_REFUSED,	/**< The job was refused by the other component. */ 
	EDG_WLL_TRANSFER_FAIL,	/**< The transfer failed for other reason than explicit refusal (eg. network timeout). */ 
};

/**
 * \fn enum edg_wll_TransferResult edg_wll_StringToTransferResult(const char *name);
 * \param[in] name		a string representing Transfer Result code (e.g. "OK")
 * \return corresponding numeric code (enum edg_wll_TransferResult)
 * \brief converts a string Transfer Result code to corresponding numeric code
 */
extern enum edg_wll_TransferResult edg_wll_StringToTransferResult(const char *name);

/**
 * \fn char *edg_wll_TransferResultToString(enum edg_wll_TransferResult code);
 * \param[in] code		a Transfer Result numeric code (enum edg_wll_TransferResult)
 * \return corresponding string (e.g. "OK")
 * \brief converts a Transfer Result numeric code to corresponding string
 */
extern char *edg_wll_TransferResultToString(enum edg_wll_TransferResult code);


/**
 * \enum edg_wll_EnQueuedResult
 * Result codes of the EnQueued event
 */
enum edg_wll_EnQueuedResult {
	EDG_WLL_ENQUEUED_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_ENQUEUED_START,	/**< The sending component has started or is about to start enqueuing the job. */ 
	EDG_WLL_ENQUEUED_OK,	/**< The job was enqueued successfully. */ 
	EDG_WLL_ENQUEUED_REFUSED,	/**< The job was refused by the other component. */ 
	EDG_WLL_ENQUEUED_FAIL,	/**< The transfer failed for other reason than explicit refusal. */ 
};

/**
 * \fn enum edg_wll_EnQueuedResult edg_wll_StringToEnQueuedResult(const char *name);
 * \param[in] name		a string representing EnQueued Result code (e.g. "OK")
 * \return corresponding numeric code (enum edg_wll_EnQueuedResult)
 * \brief converts a string EnQueued Result code to corresponding numeric code
 */
extern enum edg_wll_EnQueuedResult edg_wll_StringToEnQueuedResult(const char *name);

/**
 * \fn char *edg_wll_EnQueuedResultToString(enum edg_wll_EnQueuedResult code);
 * \param[in] code		a EnQueued Result numeric code (enum edg_wll_EnQueuedResult)
 * \return corresponding string (e.g. "OK")
 * \brief converts a EnQueued Result numeric code to corresponding string
 */
extern char *edg_wll_EnQueuedResultToString(enum edg_wll_EnQueuedResult code);


/**
 * \enum edg_wll_HelperCallSrc_role
 * Src_role codes of the HelperCall event
 */
enum edg_wll_HelperCallSrc_role {
	EDG_WLL_HELPERCALL_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_HELPERCALL_CALLING,	/**< The logging component is caller. */ 
	EDG_WLL_HELPERCALL_CALLED,	/**< The logging component is callee. */ 
};

/**
 * \fn enum edg_wll_HelperCallSrc_role edg_wll_StringToHelperCallSrc_role(const char *name);
 * \param[in] name		a string representing HelperCall Src_role code (e.g. "CALLED")
 * \return corresponding numeric code (enum edg_wll_HelperCallSrc_role)
 * \brief converts a string HelperCall Src_role code to corresponding numeric code
 */
extern enum edg_wll_HelperCallSrc_role edg_wll_StringToHelperCallSrc_role(const char *name);

/**
 * \fn char *edg_wll_HelperCallSrc_roleToString(enum edg_wll_HelperCallSrc_role code);
 * \param[in] code		a HelperCall Src_role numeric code (enum edg_wll_HelperCallSrc_role)
 * \return corresponding string (e.g. "CALLED")
 * \brief converts a HelperCall Src_role numeric code to corresponding string
 */
extern char *edg_wll_HelperCallSrc_roleToString(enum edg_wll_HelperCallSrc_role code);


/**
 * \enum edg_wll_HelperReturnSrc_role
 * Src_role codes of the HelperReturn event
 */
enum edg_wll_HelperReturnSrc_role {
	EDG_WLL_HELPERRETURN_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_HELPERRETURN_CALLING,	/**< The logging component is caller. */ 
	EDG_WLL_HELPERRETURN_CALLED,	/**< The logging component is callee. */ 
};

/**
 * \fn enum edg_wll_HelperReturnSrc_role edg_wll_StringToHelperReturnSrc_role(const char *name);
 * \param[in] name		a string representing HelperReturn Src_role code (e.g. "CALLED")
 * \return corresponding numeric code (enum edg_wll_HelperReturnSrc_role)
 * \brief converts a string HelperReturn Src_role code to corresponding numeric code
 */
extern enum edg_wll_HelperReturnSrc_role edg_wll_StringToHelperReturnSrc_role(const char *name);

/**
 * \fn char *edg_wll_HelperReturnSrc_roleToString(enum edg_wll_HelperReturnSrc_role code);
 * \param[in] code		a HelperReturn Src_role numeric code (enum edg_wll_HelperReturnSrc_role)
 * \return corresponding string (e.g. "CALLED")
 * \brief converts a HelperReturn Src_role numeric code to corresponding string
 */
extern char *edg_wll_HelperReturnSrc_roleToString(enum edg_wll_HelperReturnSrc_role code);


/**
 * \enum edg_wll_ResubmissionResult
 * Result codes of the Resubmission event
 */
enum edg_wll_ResubmissionResult {
	EDG_WLL_RESUBMISSION_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_RESUBMISSION_WILLRESUB,	/**< The job will be resubmitted (deep resubmission). */ 
	EDG_WLL_RESUBMISSION_WONTRESUB,	/**< The job will not be resubmitted. */ 
	EDG_WLL_RESUBMISSION_SHALLOW,	/**< Shallow resubmission (user payload has not started yet) */ 
};

/**
 * \fn enum edg_wll_ResubmissionResult edg_wll_StringToResubmissionResult(const char *name);
 * \param[in] name		a string representing Resubmission Result code (e.g. "WONTRESUB")
 * \return corresponding numeric code (enum edg_wll_ResubmissionResult)
 * \brief converts a string Resubmission Result code to corresponding numeric code
 */
extern enum edg_wll_ResubmissionResult edg_wll_StringToResubmissionResult(const char *name);

/**
 * \fn char *edg_wll_ResubmissionResultToString(enum edg_wll_ResubmissionResult code);
 * \param[in] code		a Resubmission Result numeric code (enum edg_wll_ResubmissionResult)
 * \return corresponding string (e.g. "WONTRESUB")
 * \brief converts a Resubmission Result numeric code to corresponding string
 */
extern char *edg_wll_ResubmissionResultToString(enum edg_wll_ResubmissionResult code);


/**
 * \enum edg_wll_DoneStatus_code
 * Status_code codes of the Done event
 */
enum edg_wll_DoneStatus_code {
	EDG_WLL_DONE_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_DONE_OK,	/**< The job terminated by itself. */ 
	EDG_WLL_DONE_FAILED,	/**< The job disappeared from LRMS. */ 
	EDG_WLL_DONE_CANCELLED,	/**< The job was cancelled by user request. */ 
};

/**
 * \fn enum edg_wll_DoneStatus_code edg_wll_StringToDoneStatus_code(const char *name);
 * \param[in] name		a string representing Done Status_code code (e.g. "FAILED")
 * \return corresponding numeric code (enum edg_wll_DoneStatus_code)
 * \brief converts a string Done Status_code code to corresponding numeric code
 */
extern enum edg_wll_DoneStatus_code edg_wll_StringToDoneStatus_code(const char *name);

/**
 * \fn char *edg_wll_DoneStatus_codeToString(enum edg_wll_DoneStatus_code code);
 * \param[in] code		a Done Status_code numeric code (enum edg_wll_DoneStatus_code)
 * \return corresponding string (e.g. "FAILED")
 * \brief converts a Done Status_code numeric code to corresponding string
 */
extern char *edg_wll_DoneStatus_codeToString(enum edg_wll_DoneStatus_code code);


/**
 * \enum edg_wll_CancelStatus_code
 * Status_code codes of the Cancel event
 */
enum edg_wll_CancelStatus_code {
	EDG_WLL_CANCEL_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_CANCEL_REQ,	/**< The request was acknowledged. */ 
	EDG_WLL_CANCEL_REFUSE,	/**< The request was declined by this component. */ 
	EDG_WLL_CANCEL_DONE,	/**< The request was completed by whole WMS. */ 
	EDG_WLL_CANCEL_ABORT,	/**< The request was refused by whole WMS. */ 
};

/**
 * \fn enum edg_wll_CancelStatus_code edg_wll_StringToCancelStatus_code(const char *name);
 * \param[in] name		a string representing Cancel Status_code code (e.g. "REFUSE")
 * \return corresponding numeric code (enum edg_wll_CancelStatus_code)
 * \brief converts a string Cancel Status_code code to corresponding numeric code
 */
extern enum edg_wll_CancelStatus_code edg_wll_StringToCancelStatus_code(const char *name);

/**
 * \fn char *edg_wll_CancelStatus_codeToString(enum edg_wll_CancelStatus_code code);
 * \param[in] code		a Cancel Status_code numeric code (enum edg_wll_CancelStatus_code)
 * \return corresponding string (e.g. "REFUSE")
 * \brief converts a Cancel Status_code numeric code to corresponding string
 */
extern char *edg_wll_CancelStatus_codeToString(enum edg_wll_CancelStatus_code code);


/**
 * \enum edg_wll_ClearReason
 * Reason codes of the Clear event
 */
enum edg_wll_ClearReason {
	EDG_WLL_CLEAR_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_CLEAR_USER,	/**< User retrieved output sandbox. */ 
	EDG_WLL_CLEAR_TIMEOUT,	/**< Timed out, resource forced purge of the sandbox. */ 
	EDG_WLL_CLEAR_NOOUTPUT,	/**< No output was generated. */ 
};

/**
 * \fn enum edg_wll_ClearReason edg_wll_StringToClearReason(const char *name);
 * \param[in] name		a string representing Clear Reason code (e.g. "TIMEOUT")
 * \return corresponding numeric code (enum edg_wll_ClearReason)
 * \brief converts a string Clear Reason code to corresponding numeric code
 */
extern enum edg_wll_ClearReason edg_wll_StringToClearReason(const char *name);

/**
 * \fn char *edg_wll_ClearReasonToString(enum edg_wll_ClearReason code);
 * \param[in] code		a Clear Reason numeric code (enum edg_wll_ClearReason)
 * \return corresponding string (e.g. "TIMEOUT")
 * \brief converts a Clear Reason numeric code to corresponding string
 */
extern char *edg_wll_ClearReasonToString(enum edg_wll_ClearReason code);


/**
 * \enum edg_wll_RegJobJobtype
 * Jobtype codes of the RegJob event
 */
enum edg_wll_RegJobJobtype {
	EDG_WLL_REGJOB_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_REGJOB_SIMPLE,	/**< The job is simple job. */ 
	EDG_WLL_REGJOB_DAG,	/**< The job is dag (containing static set of subjobs). */ 
	EDG_WLL_REGJOB_PARTITIONABLE,	/**< The job is partitionable (may become partitioned). */ 
	EDG_WLL_REGJOB_PARTITIONED,	/**< The job is partitioned (dynamically created dag). */ 
	EDG_WLL_REGJOB_COLLECTION,	/**< The job is collection (containing static set of subjobs). */ 
	EDG_WLL_REGJOB_PBS,	/**< PBS job */ 
	EDG_WLL_REGJOB_CONDOR,	/**< Condor job */ 
};

/**
 * \fn enum edg_wll_RegJobJobtype edg_wll_StringToRegJobJobtype(const char *name);
 * \param[in] name		a string representing RegJob Jobtype code (e.g. "DAG")
 * \return corresponding numeric code (enum edg_wll_RegJobJobtype)
 * \brief converts a string RegJob Jobtype code to corresponding numeric code
 */
extern enum edg_wll_RegJobJobtype edg_wll_StringToRegJobJobtype(const char *name);

/**
 * \fn char *edg_wll_RegJobJobtypeToString(enum edg_wll_RegJobJobtype code);
 * \param[in] code		a RegJob Jobtype numeric code (enum edg_wll_RegJobJobtype)
 * \return corresponding string (e.g. "DAG")
 * \brief converts a RegJob Jobtype numeric code to corresponding string
 */
extern char *edg_wll_RegJobJobtypeToString(enum edg_wll_RegJobJobtype code);


/**
 * \enum edg_wll_PBSResourceUsageUsage
 * Usage codes of the PBSResourceUsage event
 */
enum edg_wll_PBSResourceUsageUsage {
	EDG_WLL_PBSRESOURCEUSAGE_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_PBSRESOURCEUSAGE_REQUESTED,	/**< Requested value */ 
	EDG_WLL_PBSRESOURCEUSAGE_USED,	/**< Consumed quantity */ 
};

/**
 * \fn enum edg_wll_PBSResourceUsageUsage edg_wll_StringToPBSResourceUsageUsage(const char *name);
 * \param[in] name		a string representing PBSResourceUsage Usage code (e.g. "USED")
 * \return corresponding numeric code (enum edg_wll_PBSResourceUsageUsage)
 * \brief converts a string PBSResourceUsage Usage code to corresponding numeric code
 */
extern enum edg_wll_PBSResourceUsageUsage edg_wll_StringToPBSResourceUsageUsage(const char *name);

/**
 * \fn char *edg_wll_PBSResourceUsageUsageToString(enum edg_wll_PBSResourceUsageUsage code);
 * \param[in] code		a PBSResourceUsage Usage numeric code (enum edg_wll_PBSResourceUsageUsage)
 * \return corresponding string (e.g. "USED")
 * \brief converts a PBSResourceUsage Usage numeric code to corresponding string
 */
extern char *edg_wll_PBSResourceUsageUsageToString(enum edg_wll_PBSResourceUsageUsage code);


/**
 * \enum edg_wll_CondorRejectStatus_code
 * Status_code codes of the CondorReject event
 */
enum edg_wll_CondorRejectStatus_code {
	EDG_WLL_CONDORREJECT_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_CONDORREJECT_NOMATCH,	/**< No match found */ 
	EDG_WLL_CONDORREJECT_OTHER,	/**< Other reason */ 
};

/**
 * \fn enum edg_wll_CondorRejectStatus_code edg_wll_StringToCondorRejectStatus_code(const char *name);
 * \param[in] name		a string representing CondorReject Status_code code (e.g. "OTHER")
 * \return corresponding numeric code (enum edg_wll_CondorRejectStatus_code)
 * \brief converts a string CondorReject Status_code code to corresponding numeric code
 */
extern enum edg_wll_CondorRejectStatus_code edg_wll_StringToCondorRejectStatus_code(const char *name);

/**
 * \fn char *edg_wll_CondorRejectStatus_codeToString(enum edg_wll_CondorRejectStatus_code code);
 * \param[in] code		a CondorReject Status_code numeric code (enum edg_wll_CondorRejectStatus_code)
 * \return corresponding string (e.g. "OTHER")
 * \brief converts a CondorReject Status_code numeric code to corresponding string
 */
extern char *edg_wll_CondorRejectStatus_codeToString(enum edg_wll_CondorRejectStatus_code code);


/**
 * \enum edg_wll_CondorGlobusSubmitStatus_code
 * Status_code codes of the CondorGlobusSubmit event
 */
enum edg_wll_CondorGlobusSubmitStatus_code {
	EDG_WLL_CONDORGLOBUSSUBMIT_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_CONDORGLOBUSSUBMIT_OK,	/**< The job was submitted successfully */ 
	EDG_WLL_CONDORGLOBUSSUBMIT_FAIL,	/**< Job submission failed */ 
};

/**
 * \fn enum edg_wll_CondorGlobusSubmitStatus_code edg_wll_StringToCondorGlobusSubmitStatus_code(const char *name);
 * \param[in] name		a string representing CondorGlobusSubmit Status_code code (e.g. "FAIL")
 * \return corresponding numeric code (enum edg_wll_CondorGlobusSubmitStatus_code)
 * \brief converts a string CondorGlobusSubmit Status_code code to corresponding numeric code
 */
extern enum edg_wll_CondorGlobusSubmitStatus_code edg_wll_StringToCondorGlobusSubmitStatus_code(const char *name);

/**
 * \fn char *edg_wll_CondorGlobusSubmitStatus_codeToString(enum edg_wll_CondorGlobusSubmitStatus_code code);
 * \param[in] code		a CondorGlobusSubmit Status_code numeric code (enum edg_wll_CondorGlobusSubmitStatus_code)
 * \return corresponding string (e.g. "FAIL")
 * \brief converts a CondorGlobusSubmit Status_code numeric code to corresponding string
 */
extern char *edg_wll_CondorGlobusSubmitStatus_codeToString(enum edg_wll_CondorGlobusSubmitStatus_code code);


/**
 * \enum edg_wll_CondorGlobusResourceStatus_code
 * Status_code codes of the CondorGlobusResource event
 */
enum edg_wll_CondorGlobusResourceStatus_code {
	EDG_WLL_CONDORGLOBUSRESOURCE_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_CONDORGLOBUSRESOURCE_UP,	/**< Resource UP */ 
	EDG_WLL_CONDORGLOBUSRESOURCE_DOWN,	/**< Resource DOWM */ 
};

/**
 * \fn enum edg_wll_CondorGlobusResourceStatus_code edg_wll_StringToCondorGlobusResourceStatus_code(const char *name);
 * \param[in] name		a string representing CondorGlobusResource Status_code code (e.g. "DOWN")
 * \return corresponding numeric code (enum edg_wll_CondorGlobusResourceStatus_code)
 * \brief converts a string CondorGlobusResource Status_code code to corresponding numeric code
 */
extern enum edg_wll_CondorGlobusResourceStatus_code edg_wll_StringToCondorGlobusResourceStatus_code(const char *name);

/**
 * \fn char *edg_wll_CondorGlobusResourceStatus_codeToString(enum edg_wll_CondorGlobusResourceStatus_code code);
 * \param[in] code		a CondorGlobusResource Status_code numeric code (enum edg_wll_CondorGlobusResourceStatus_code)
 * \return corresponding string (e.g. "DOWN")
 * \brief converts a CondorGlobusResource Status_code numeric code to corresponding string
 */
extern char *edg_wll_CondorGlobusResourceStatus_codeToString(enum edg_wll_CondorGlobusResourceStatus_code code);



/**
 * common fields of all event types: 
 */

#define _EDG_WLL_EVENT_COMMON \
	edg_wll_EventCode		type;\
	struct timeval		timestamp;	\
	struct timeval		arrived;	\
	char *		host;	\
	int		level;	\
	int		priority;	\
	edg_wlc_JobId		jobId;	\
	char *		seqcode;	\
	char *		user;	\
	edg_wll_Source		source;	\
	char *		src_instance;	\


/**
 * \typedef edg_wll_AnyEvent
 * common structure definition contained in all types of the events
 */
typedef struct _edg_wll_AnyEvent {
_EDG_WLL_EVENT_COMMON
} edg_wll_AnyEvent;


/**
 *  Event types specific structures: 
 */

/**
* \typedef edg_wll_TransferEvent
* structure definition for Transfer event 
*/
typedef struct _edg_wll_TransferEvent {
_EDG_WLL_EVENT_COMMON
	edg_wll_Source	destination;	/**< Destination where the job is being transfered to. */
	char *	dest_host;	/**< Hostname of server that takes over control of the job. */
	char *	dest_instance;	/**< Service (instance) that takes over control of the job. */
	char *	job;	/**< Job description in receiver's language. */
	enum edg_wll_TransferResult	result;	/**< Result code of the transfer attempt (START, OK, REFUSED or FAIL). */
	char *	reason;	/**< Detailed description of the transfer, especially reason of failure. */
	char *	dest_jobid;	/**< Job id as assigned by the receiving software component. */
} edg_wll_TransferEvent;

/**
* \typedef edg_wll_AcceptedEvent
* structure definition for Accepted event 
*/
typedef struct _edg_wll_AcceptedEvent {
_EDG_WLL_EVENT_COMMON
	edg_wll_Source	from;	/**< The software component  the job was received from. */
	char *	from_host;	/**< Hostname of the component the job was received from. */
	char *	from_instance;	/**< Instance of the component the job was received from. */
	char *	local_jobid;	/**< New job id as assigned by the receiving component. */
} edg_wll_AcceptedEvent;

/**
* \typedef edg_wll_RefusedEvent
* structure definition for Refused event 
*/
typedef struct _edg_wll_RefusedEvent {
_EDG_WLL_EVENT_COMMON
	edg_wll_Source	from;	/**< The software component that tried to send the job. */
	char *	from_host;	/**< Hostname of the component that tried to send the job. */
	char *	from_instance;	/**< Instance of the component that tried to send the job. */
	char *	reason;	/**< Description of the reason why the job was refused. */
} edg_wll_RefusedEvent;

/**
* \typedef edg_wll_EnQueuedEvent
* structure definition for EnQueued event 
*/
typedef struct _edg_wll_EnQueuedEvent {
_EDG_WLL_EVENT_COMMON
	char *	queue;	/**< Queue into which the job has been stored for retrieval by another component. */
	char *	job;	/**< Job description in the receiver's language. */
	enum edg_wll_EnQueuedResult	result;	/**< Result code of the attempt to put job into the queue (START, OK, REFUSED or FAIL). */
	char *	reason;	/**< Detailed description of the attempt to enqueue the job, especially the reason of failure. */
} edg_wll_EnQueuedEvent;

/**
* \typedef edg_wll_DeQueuedEvent
* structure definition for DeQueued event 
*/
typedef struct _edg_wll_DeQueuedEvent {
_EDG_WLL_EVENT_COMMON
	char *	queue;	/**< Name of the queue the job was obtained from. */
	char *	local_jobid;	/**< New job id as assigned by the retreiving component. */
} edg_wll_DeQueuedEvent;

/**
* \typedef edg_wll_HelperCallEvent
* structure definition for HelperCall event 
*/
typedef struct _edg_wll_HelperCallEvent {
_EDG_WLL_EVENT_COMMON
	char *	helper_name;	/**< Name of the called helper component. */
	char *	helper_params;	/**< Parameters of the call to the helper component. */
	enum edg_wll_HelperCallSrc_role	src_role;	/**< The role the event sender is playing in the helper call (CALLING or CALLEE). */
} edg_wll_HelperCallEvent;

/**
* \typedef edg_wll_HelperReturnEvent
* structure definition for HelperReturn event 
*/
typedef struct _edg_wll_HelperReturnEvent {
_EDG_WLL_EVENT_COMMON
	char *	helper_name;	/**< Name of the called helper component. */
	char *	retval;	/**< Data returned by the call to the helper component. */
	enum edg_wll_HelperReturnSrc_role	src_role;	/**< The role the event sender is playing in the helper call (CALLING or CALLEE). */
} edg_wll_HelperReturnEvent;

/**
* \typedef edg_wll_RunningEvent
* structure definition for Running event 
*/
typedef struct _edg_wll_RunningEvent {
_EDG_WLL_EVENT_COMMON
	char *	node;	/**< Worker node on which the job executable is being run. */
} edg_wll_RunningEvent;

/**
* \typedef edg_wll_ResubmissionEvent
* structure definition for Resubmission event 
*/
typedef struct _edg_wll_ResubmissionEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_ResubmissionResult	result;	/**< Result code of the resubmission decision (WILLRESUB or WONTRESUB or SHALLOW). */
	char *	reason;	/**< Reason why the job will or will not be resubmitted. */
	char *	tag;	/**< Value of the attribute on which the decision to resubmit the job was based. */
} edg_wll_ResubmissionEvent;

/**
* \typedef edg_wll_DoneEvent
* structure definition for Done event 
*/
typedef struct _edg_wll_DoneEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_DoneStatus_code	status_code;	/**< Reason code for the termination of the job (OK, FAILED or CANCELLED). */
	char *	reason;	/**< Detailed description why the job was terminated. */
	int	exit_code;	/**< Exit code of the job's process. */
} edg_wll_DoneEvent;

/**
* \typedef edg_wll_CancelEvent
* structure definition for Cancel event 
*/
typedef struct _edg_wll_CancelEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_CancelStatus_code	status_code;	/**< Classification of the attempt to cancel the job (REQ, REFUSE, DONE or ABORT). */
	char *	reason;	/**< Detailed description of the attempt to cancel the job, especially the reason of failure. */
} edg_wll_CancelEvent;

/**
* \typedef edg_wll_AbortEvent
* structure definition for Abort event 
*/
typedef struct _edg_wll_AbortEvent {
_EDG_WLL_EVENT_COMMON
	char *	reason;	/**< Reason why the job was aborted by the system. */
} edg_wll_AbortEvent;

/**
* \typedef edg_wll_ClearEvent
* structure definition for Clear event 
*/
typedef struct _edg_wll_ClearEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_ClearReason	reason;	/**< Description of the reason why the job was cleared and the output sandbox removed (USER, TIMEOUT or NOOUTPUT). */
} edg_wll_ClearEvent;

/**
* \typedef edg_wll_PurgeEvent
* structure definition for Purge event 
*/
typedef struct _edg_wll_PurgeEvent {
_EDG_WLL_EVENT_COMMON
} edg_wll_PurgeEvent;

/**
* \typedef edg_wll_MatchEvent
* structure definition for Match event 
*/
typedef struct _edg_wll_MatchEvent {
_EDG_WLL_EVENT_COMMON
	char *	dest_id;	/**< Identification of the queue on the CE that the job could be send to. */
} edg_wll_MatchEvent;

/**
* \typedef edg_wll_PendingEvent
* structure definition for Pending event 
*/
typedef struct _edg_wll_PendingEvent {
_EDG_WLL_EVENT_COMMON
	char *	reason;	/**< Description why the matching CE for the job was not found (yet). */
} edg_wll_PendingEvent;

/**
* \typedef edg_wll_RegJobEvent
* structure definition for RegJob event 
*/
typedef struct _edg_wll_RegJobEvent {
_EDG_WLL_EVENT_COMMON
	char *	jdl;	/**< Job description of the job being registered. */
	char *	ns;	/**< NetworkServer handling the newly registered job. */
	edg_wlc_JobId	parent;	/**< Grid job id of the parent job registering this new one. */
	enum edg_wll_RegJobJobtype	jobtype;	/**< Type of the job being registered (SIMPLE, DAG, PARTITIONABLE or PARTITIONED). */
	int	nsubjobs;	/**< Number of subjobs this job plans to spawn. */
	char *	seed;	/**< Seed for subjob id generation. */
} edg_wll_RegJobEvent;

/**
* \typedef edg_wll_ChkptEvent
* structure definition for Chkpt event 
*/
typedef struct _edg_wll_ChkptEvent {
_EDG_WLL_EVENT_COMMON
	char *	tag;	/**< Application specific checkpoint tag. */
	char *	classad;	/**< Application specific checkpoint value. */
} edg_wll_ChkptEvent;

/**
* \typedef edg_wll_ListenerEvent
* structure definition for Listener event 
*/
typedef struct _edg_wll_ListenerEvent {
_EDG_WLL_EVENT_COMMON
	char *	svc_name;	/**< Name of the port instance for interactive job control. */
	char *	svc_host;	/**< Hostname of the interactive job controller. */
	uint16_t	svc_port;	/**< Port number of the interactive job controller. */
} edg_wll_ListenerEvent;

/**
* \typedef edg_wll_CurDescrEvent
* structure definition for CurDescr event 
*/
typedef struct _edg_wll_CurDescrEvent {
_EDG_WLL_EVENT_COMMON
	char *	descr;	/**< Description of the current job transformation (output of the helper). */
} edg_wll_CurDescrEvent;

/**
* \typedef edg_wll_UserTagEvent
* structure definition for UserTag event 
*/
typedef struct _edg_wll_UserTagEvent {
_EDG_WLL_EVENT_COMMON
	char *	name;	/**< Arbitrary user tag name. */
	char *	value;	/**< Arbitrary user tag value. */
} edg_wll_UserTagEvent;

/**
* \typedef edg_wll_ChangeACLEvent
* structure definition for ChangeACL event 
*/
typedef struct _edg_wll_ChangeACLEvent {
_EDG_WLL_EVENT_COMMON
	char *	user_id;	/**< DN or VOMS parameter (in format VO:group). */
	int	user_id_type;	/**< Type of information given in user_id (DN or VOMS). */
	int	permission;	/**< ACL permission to change (currently only READ). */
	int	permission_type;	/**< Type of permission requested ('allow', 'deny'). */
	int	operation;	/**< Operation requested to perform with ACL (add, remove). */
} edg_wll_ChangeACLEvent;

/**
* \typedef edg_wll_NotificationEvent
* structure definition for Notification event 
*/
typedef struct _edg_wll_NotificationEvent {
_EDG_WLL_EVENT_COMMON
	edg_wll_NotifId	notifId;	/**< Notification id. */
	char *	owner;	/**< Identification of the job owner (certificate subject). */
	char *	dest_host;	/**< Hostname the notification is sent to. */
	uint16_t	dest_port;	/**< Port number the notification is sent to. */
	char *	jobstat;	/**< Status of the job (the notification content). */
} edg_wll_NotificationEvent;

/**
* \typedef edg_wll_ResourceUsageEvent
* structure definition for ResourceUsage event 
*/
typedef struct _edg_wll_ResourceUsageEvent {
_EDG_WLL_EVENT_COMMON
	char *	resource;	/**< Resource's name. */
	double	quantity;	/**< Resources's quantity (how much). */
	char *	unit;	/**< Units (sec, kB, etc.). */
} edg_wll_ResourceUsageEvent;

/**
* \typedef edg_wll_ReallyRunningEvent
* structure definition for ReallyRunning event 
*/
typedef struct _edg_wll_ReallyRunningEvent {
_EDG_WLL_EVENT_COMMON
	char *	wn_seq;	/**< Sequence code on the worker node. */
} edg_wll_ReallyRunningEvent;

/**
* \typedef edg_wll_SuspendEvent
* structure definition for Suspend event 
*/
typedef struct _edg_wll_SuspendEvent {
_EDG_WLL_EVENT_COMMON
	char *	reason;	/**< Reason for the suspend. */
} edg_wll_SuspendEvent;

/**
* \typedef edg_wll_ResumeEvent
* structure definition for Resume event 
*/
typedef struct _edg_wll_ResumeEvent {
_EDG_WLL_EVENT_COMMON
	char *	reason;	/**< Reason for the resume. */
} edg_wll_ResumeEvent;

/**
* \typedef edg_wll_CollectionStateEvent
* structure definition for CollectionState event 
*/
typedef struct _edg_wll_CollectionStateEvent {
_EDG_WLL_EVENT_COMMON
	char *	state;	/**< New collection state. */
	int	done_code;	/**< In case of (state == Done) contains done code  */
	char *	histogram;	/**< User readable histogram; useful for debugging. */
	edg_wlc_JobId	child;	/**< JobId of subjob, which triggered the state change.	 */
	char *	child_event;	/**< Event which triggered the state change.	 */
} edg_wll_CollectionStateEvent;

/**
* \typedef edg_wll_PBSQueuedEvent
* structure definition for PBSQueued event 
*/
typedef struct _edg_wll_PBSQueuedEvent {
_EDG_WLL_EVENT_COMMON
	char *	queue;	/**< Queue name */
	char *	owner;	/**< Job owner */
	char *	name;	/**< Job name */
} edg_wll_PBSQueuedEvent;

/**
* \typedef edg_wll_PBSMatchEvent
* structure definition for PBSMatch event 
*/
typedef struct _edg_wll_PBSMatchEvent {
_EDG_WLL_EVENT_COMMON
	char *	dest_host;	/**< Aka exec_host */
} edg_wll_PBSMatchEvent;

/**
* \typedef edg_wll_PBSPendingEvent
* structure definition for PBSPending event 
*/
typedef struct _edg_wll_PBSPendingEvent {
_EDG_WLL_EVENT_COMMON
	char *	reason;	/**< Reasons of job pendation or errors */
} edg_wll_PBSPendingEvent;

/**
* \typedef edg_wll_PBSRunEvent
* structure definition for PBSRun event 
*/
typedef struct _edg_wll_PBSRunEvent {
_EDG_WLL_EVENT_COMMON
	char *	scheduler;	/**< Scheduler ID */
	char *	dest_host;	/**< Where to run the job */
	int	pid;	/**< Actual process ID */
} edg_wll_PBSRunEvent;

/**
* \typedef edg_wll_PBSRerunEvent
* structure definition for PBSRerun event 
*/
typedef struct _edg_wll_PBSRerunEvent {
_EDG_WLL_EVENT_COMMON
} edg_wll_PBSRerunEvent;

/**
* \typedef edg_wll_PBSDoneEvent
* structure definition for PBSDone event 
*/
typedef struct _edg_wll_PBSDoneEvent {
_EDG_WLL_EVENT_COMMON
	int	exit_status;	/**< Exit status */
} edg_wll_PBSDoneEvent;

/**
* \typedef edg_wll_PBSDequeuedEvent
* structure definition for PBSDequeued event 
*/
typedef struct _edg_wll_PBSDequeuedEvent {
_EDG_WLL_EVENT_COMMON
} edg_wll_PBSDequeuedEvent;

/**
* \typedef edg_wll_PBSResourceUsageEvent
* structure definition for PBSResourceUsage event 
*/
typedef struct _edg_wll_PBSResourceUsageEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_PBSResourceUsageUsage	usage;	/**< Type of record */
	char *	name;	/**< Name of resource */
	double	quantity;	/**< The quantity */
	char *	unit;	/**< Units (sec, kB, etc.) */
} edg_wll_PBSResourceUsageEvent;

/**
* \typedef edg_wll_PBSErrorEvent
* structure definition for PBSError event 
*/
typedef struct _edg_wll_PBSErrorEvent {
_EDG_WLL_EVENT_COMMON
	char *	error_desc;	/**< Error reason */
} edg_wll_PBSErrorEvent;

/**
* \typedef edg_wll_CondorMatchEvent
* structure definition for CondorMatch event 
*/
typedef struct _edg_wll_CondorMatchEvent {
_EDG_WLL_EVENT_COMMON
	char *	owner;	/**< Owner */
	char *	matched_host;	/**< Matched host */
	char *	preempting;	/**< Preempting */
} edg_wll_CondorMatchEvent;

/**
* \typedef edg_wll_CondorRejectEvent
* structure definition for CondorReject event 
*/
typedef struct _edg_wll_CondorRejectEvent {
_EDG_WLL_EVENT_COMMON
	char *	owner;	/**< Owner */
	enum edg_wll_CondorRejectStatus_code	status_code;	/**< Reason code for the rejection of the job */
} edg_wll_CondorRejectEvent;

/**
* \typedef edg_wll_CondorRunEvent
* structure definition for CondorRun event 
*/
typedef struct _edg_wll_CondorRunEvent {
_EDG_WLL_EVENT_COMMON
	char *	condor_id;	/**< Condor ID */
	char *	universe;	/**< Condor Universe */
	char *	scheduler;	/**< Scheduler ID */
	char *	dest_host;	/**< Where to run the job */
	int	pid;	/**< Actual process ID */
} edg_wll_CondorRunEvent;

/**
* \typedef edg_wll_CondorChkptEvent
* structure definition for CondorChkpt event 
*/
typedef struct _edg_wll_CondorChkptEvent {
_EDG_WLL_EVENT_COMMON
	char *	info;	/**< Some useful info */
} edg_wll_CondorChkptEvent;

/**
* \typedef edg_wll_CondorEvictEvent
* structure definition for CondorEvict event 
*/
typedef struct _edg_wll_CondorEvictEvent {
_EDG_WLL_EVENT_COMMON
	int	shadow_exit_status;	/**< Condor shadow exit status */
	char *	reason;	/**< reason */
} edg_wll_CondorEvictEvent;

/**
* \typedef edg_wll_CondorTerminateEvent
* structure definition for CondorTerminate event 
*/
typedef struct _edg_wll_CondorTerminateEvent {
_EDG_WLL_EVENT_COMMON
	int	shadow_exit_status;	/**< Condor shadow exit status */
	char *	reason;	/**< reason */
} edg_wll_CondorTerminateEvent;

/**
* \typedef edg_wll_CondorAbortEvent
* structure definition for CondorAbort event 
*/
typedef struct _edg_wll_CondorAbortEvent {
_EDG_WLL_EVENT_COMMON
	int	shadow_exit_status;	/**< Condor shadow exit status */
	char *	reason;	/**< reason */
} edg_wll_CondorAbortEvent;

/**
* \typedef edg_wll_CondorSuspendEvent
* structure definition for CondorSuspend event 
*/
typedef struct _edg_wll_CondorSuspendEvent {
_EDG_WLL_EVENT_COMMON
	int	shadow_exit_status;	/**< Condor shadow exit status */
	char *	reason;	/**< reason */
} edg_wll_CondorSuspendEvent;

/**
* \typedef edg_wll_CondorUnsuspendEvent
* structure definition for CondorUnsuspend event 
*/
typedef struct _edg_wll_CondorUnsuspendEvent {
_EDG_WLL_EVENT_COMMON
	int	shadow_exit_status;	/**< Condor shadow exit status */
	char *	reason;	/**< reason */
} edg_wll_CondorUnsuspendEvent;

/**
* \typedef edg_wll_CondorHoldEvent
* structure definition for CondorHold event 
*/
typedef struct _edg_wll_CondorHoldEvent {
_EDG_WLL_EVENT_COMMON
	int	shadow_exit_status;	/**< Condor shadow exit status */
	char *	reason;	/**< reason */
} edg_wll_CondorHoldEvent;

/**
* \typedef edg_wll_CondorReleaseEvent
* structure definition for CondorRelease event 
*/
typedef struct _edg_wll_CondorReleaseEvent {
_EDG_WLL_EVENT_COMMON
	int	shadow_exit_status;	/**< Condor shadow exit status */
	char *	reason;	/**< reason */
} edg_wll_CondorReleaseEvent;

/**
* \typedef edg_wll_CondorNodeExecuteEvent
* structure definition for CondorNodeExecute event 
*/
typedef struct _edg_wll_CondorNodeExecuteEvent {
_EDG_WLL_EVENT_COMMON
	char *	info;	/**< Some useful info */
} edg_wll_CondorNodeExecuteEvent;

/**
* \typedef edg_wll_CondorNodeTerminateEvent
* structure definition for CondorNodeTerminate event 
*/
typedef struct _edg_wll_CondorNodeTerminateEvent {
_EDG_WLL_EVENT_COMMON
	char *	info;	/**< Some useful info */
} edg_wll_CondorNodeTerminateEvent;

/**
* \typedef edg_wll_CondorPostSctiptTerminateEvent
* structure definition for CondorPostSctiptTerminate event 
*/
typedef struct _edg_wll_CondorPostSctiptTerminateEvent {
_EDG_WLL_EVENT_COMMON
	char *	info;	/**< Some useful info */
} edg_wll_CondorPostSctiptTerminateEvent;

/**
* \typedef edg_wll_CondorGlobusSubmitEvent
* structure definition for CondorGlobusSubmit event 
*/
typedef struct _edg_wll_CondorGlobusSubmitEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_CondorGlobusSubmitStatus_code	status_code;	/**<  */
	char *	info;	/**< Some useful info */
} edg_wll_CondorGlobusSubmitEvent;

/**
* \typedef edg_wll_CondorGlobusResourceEvent
* structure definition for CondorGlobusResource event 
*/
typedef struct _edg_wll_CondorGlobusResourceEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_CondorGlobusResourceStatus_code	status_code;	/**< Status code */
	char *	info;	/**< Some useful info */
} edg_wll_CondorGlobusResourceEvent;

#undef _EDG_WLL_EVENT_COMMON


/**
 * \union edg_wll_Event
 * \brief All event types union
 */
typedef union _edg_wll_Event {
        edg_wll_EventCode           type; /* it is probably never used */
        edg_wll_AnyEvent            any;
	edg_wll_TransferEvent	transfer;
	edg_wll_AcceptedEvent	accepted;
	edg_wll_RefusedEvent	refused;
	edg_wll_EnQueuedEvent	enQueued;
	edg_wll_DeQueuedEvent	deQueued;
	edg_wll_HelperCallEvent	helperCall;
	edg_wll_HelperReturnEvent	helperReturn;
	edg_wll_RunningEvent	running;
	edg_wll_ResubmissionEvent	resubmission;
	edg_wll_DoneEvent	done;
	edg_wll_CancelEvent	cancel;
	edg_wll_AbortEvent	abort;
	edg_wll_ClearEvent	clear;
	edg_wll_PurgeEvent	purge;
	edg_wll_MatchEvent	match;
	edg_wll_PendingEvent	pending;
	edg_wll_RegJobEvent	regJob;
	edg_wll_ChkptEvent	chkpt;
	edg_wll_ListenerEvent	listener;
	edg_wll_CurDescrEvent	curDescr;
	edg_wll_UserTagEvent	userTag;
	edg_wll_ChangeACLEvent	changeACL;
	edg_wll_NotificationEvent	notification;
	edg_wll_ResourceUsageEvent	resourceUsage;
	edg_wll_ReallyRunningEvent	reallyRunning;
	edg_wll_SuspendEvent	suspend;
	edg_wll_ResumeEvent	resume;
	edg_wll_CollectionStateEvent	collectionState;
	edg_wll_PBSQueuedEvent	PBSQueued;
	edg_wll_PBSMatchEvent	PBSMatch;
	edg_wll_PBSPendingEvent	PBSPending;
	edg_wll_PBSRunEvent	PBSRun;
	edg_wll_PBSRerunEvent	PBSRerun;
	edg_wll_PBSDoneEvent	PBSDone;
	edg_wll_PBSDequeuedEvent	PBSDequeued;
	edg_wll_PBSResourceUsageEvent	PBSResourceUsage;
	edg_wll_PBSErrorEvent	PBSError;
	edg_wll_CondorMatchEvent	condorMatch;
	edg_wll_CondorRejectEvent	condorReject;
	edg_wll_CondorRunEvent	condorRun;
	edg_wll_CondorChkptEvent	condorChkpt;
	edg_wll_CondorEvictEvent	condorEvict;
	edg_wll_CondorTerminateEvent	condorTerminate;
	edg_wll_CondorAbortEvent	condorAbort;
	edg_wll_CondorSuspendEvent	condorSuspend;
	edg_wll_CondorUnsuspendEvent	condorUnsuspend;
	edg_wll_CondorHoldEvent	condorHold;
	edg_wll_CondorReleaseEvent	condorRelease;
	edg_wll_CondorNodeExecuteEvent	condorNodeExecute;
	edg_wll_CondorNodeTerminateEvent	condorNodeTerminate;
	edg_wll_CondorPostSctiptTerminateEvent	condorPostSctiptTerminate;
	edg_wll_CondorGlobusSubmitEvent	condorGlobusSubmit;
	edg_wll_CondorGlobusResourceEvent	condorGlobusResource;
} edg_wll_Event;


/**
 * Initialise an event structure
 * \return pointer to initialised event structure
 */
extern edg_wll_Event *edg_wll_InitEvent(edg_wll_EventCode eventcode);


/**
 * Free the contents of event structure
 * \param[in] event 		structure to be freed
 * \warning As event structures are likely to be allocated in arrays,
 *      the structure itself is not freed.
 *      Its the responsibility of the caller to call free(event)
 *      if appropriate.
 */
void edg_wll_FreeEvent(
	edg_wll_Event * event
);

/*
 *@} end of group
 */

#ifdef __cplusplus
}
#endif

#endif /* __EDG_WORKLOAD_LOGGING_CLIENT_EVENTS_H__ */
